/*   SCCS Id: @(#)tile2bmp.c   3.3     95/01/26                     */
/*   Copyright (c) NetHack PC Development Team 1993, 1994, 1995     */
/*   NetHack may be freely redistributed.  See license for details. */

/*
 * This creates a tile2bmp.exe
 *
 * tile2bmp.exe -i generates 'index'
 * tile2bmp.exe -h generates 'allegfx.h'
 * tile2bmp.exe -b generates all the *.bmp
 *
 * Edit History:
 *
 *      Initial Creation                        K.Youngblood    ??/??/??
 *      added 32x32 text tile support		W.Cheung	00/06/19
 *	added direct write to files		W.Cheung	00/06/21
 *
 */

#ifndef INDEX
#define alleg_mouse_unused
#define alleg_timer_unused
#define alleg_keyboard_unused
#define alleg_joystick_unused
#define alleg_sound_unused
#define alleg_gui_unused
#define alleg_datafile_unused

#include <allegro.h>
#endif
#include "hack.h"
#include "pcvideo.h"
#include "tile.h"
#include "pctiles.h"

#include <ctype.h>
#include <dos.h>
#ifndef MONITOR_HEAP
#include <stdlib.h>
#endif
#include <time.h>
extern char *FDECL(tilename, (int, int));

#define Fprintf (void) fprintf
#define Fclose  (void) fclose

static int num_colors;
static pixel pixels[MAX_TILE_Y][MAX_TILE_X];
static BITMAP *bigtile_bmp;

static struct tibhdr_struct tibheader;

/* [WAC] Use the 32x32x262 color tiles */

static const char 
    *tilefiles[] = {    "../win/share/mon32.txt",
                        "../win/share/obj32.txt",
                        "../win/share/oth32.txt"};

static const char
    *Dont_Edit_Code =
	"/* This source file is generated by 'tile2bmp'.  Do not edit. */\n",
    *Dont_Edit_Index =
	"# This index file was generated by 'tile2bmp'.\n";

static int tilecount;
static int filenum;
static int paletteflag;

#define TRANS_COLOR 0xff00ff

static const char
    *bigtile_file = "../src/32tile.bmp",
    *index_file = "../src/index",
    *allegfx_file = "../include/allegfx.h";

#ifndef INDEX

void
make_bitmap(pixels, tilecount, trans, filename, make_big)
pixel (*pixels)[MAX_TILE_X];
int tilecount;
int trans;
char *filename;
boolean make_big;
{
        int i, j, x, y;
        long color, color2;
        char cmd[60];
        BITMAP *tilebmp;

	/* Check for the right number of cols/rows */
        if (tile_x != 32 || tile_y != 32) {
                Fprintf(stderr, "Wrong size tiles (%d x %d)\n", tile_x, tile_y);
                exit(EXIT_FAILURE);
        }


        tilebmp = create_bitmap(32, 32);

        color2 = 0L;
        color2 |= 108;
        color2 |= (108) << 8;
        color2 |= (71) << 16;

        /* load the origional tile */

        for (j = 0; j < tile_y; j++) {
                for (i = 0; i < tile_x; i++) 
                {
                        color = 0L;
                        color |= (pixels[j][i].b & 0x0000ff);
                        color |= (pixels[j][i].g & 0x0000ff) << 8;
                        color |= (pixels[j][i].r & 0x0000ff) << 16;
                        putpixel(tilebmp, i, j, color);
                }
        }

#if 0
        drawing_mode(DRAW_MODE_TRANS, tilebmp, 0, 0);
        set_trans_blender(255, 255, 255, 127);

        for(y = 0; y < tile_y; y++)
                for(x = 1; x < tile_x; x += 2)
                {
                        color=getpixel(tilebmp, min(x+1,tile_x - 2),y);
                        putpixel(tilebmp, x, y, color);
                }
        /* interpolate horizontal lines */

        for(x = 0; x < tile_x; x++)
                for(y = 1; y < tile_y; y += 2)
                {
                        color = getpixel(tilebmp, x, min(y+1,tile_y - 2));
                        putpixel(tilebmp, x, y, color);
                }
#endif

        drawing_mode(DRAW_MODE_SOLID, tilebmp, 0, 0);

        if(trans)
        {
		/* WAC slower, but more accurate */
		for (x = 0; x < tile_x; x++)
		    for (y = 0; y < tile_y; y++)
	                if(getpixel(tilebmp, x, y) == color2)
        	                putpixel(tilebmp,x,y, TRANS_COLOR);
#if 0
                if(getpixel(tilebmp, 0, 0) == color2)
                        floodfill(tilebmp,0,0, TRANS_COLOR);
                if(getpixel(tilebmp, 0, 31) == color2)
                        floodfill(tilebmp,0,31, TRANS_COLOR);
                if(getpixel(tilebmp, 31, 0) == color2)
                        floodfill(tilebmp,31,0, TRANS_COLOR);
                if(getpixel(tilebmp, 31, 31) == color2)
                        floodfill(tilebmp,31,31, TRANS_COLOR);
#endif
        }

	if (!make_big) {
            save_bitmap(filename, tilebmp, 0);
	} else {
	    int col = (int)(tilecount % TILES_PER_COL);
	    int row = (int)(tilecount / TILES_PER_ROW);
#ifdef DEBUG
	    Fprintf(stderr, "col: %i row: %i\n", col, row);
#endif
	    blit(tilebmp, bigtile_bmp, 0, 0, col * tile_x, row * tile_y, tile_x, tile_y);
	}
        destroy_bitmap(tilebmp);
}
#endif



int
main(argc, argv)
int argc;
char *argv[];
{
        int i;
        struct tm *newtime;
        time_t aclock;
        char *paletteptr;
        char cmd2[60];
        char filename[60];
        char util_mode;
        FILE *ofp; /* Output for -i, -h */

        if (argc != 2) {
	    	Fprintf(stderr, "Bad arg count (%d).\n", argc-1);
	    	(void) fflush(stderr);
                exit(EXIT_FAILURE);
        }
        
        util_mode = argv[1][1];
        
        if (util_mode == 'I') util_mode = 'i';
        if (util_mode == 'H') util_mode = 'h';
        if (util_mode == 'F') util_mode = 'f';
        if (util_mode == 'B') util_mode = 'b';
        
        if (util_mode != 'i' && util_mode != 'h' && util_mode != 'f' 
			     && util_mode != 'b') {
	    	Fprintf(stderr, "Unknown option '-%c'.\n Use -i, -h -f or -b.\n", 
	    					util_mode);
	    	(void) fflush(stderr);
                exit(EXIT_FAILURE);        	
        }

        set_color_depth(24);

        time(&aclock);
        newtime = localtime(&aclock);

        tilecount = 0;
        paletteflag = 0;        
        filenum = 0;
        
        /* Open file (if needed) */
        if (util_mode == 'i') {
	    if (!(ofp = fopen(index_file, WRTMODE))) {
		perror(index_file);
		exit(EXIT_FAILURE);
	    }
	    Fprintf(ofp,Dont_Edit_Index);   
        } else if (util_mode == 'h') {
	    if (!(ofp = fopen(allegfx_file, WRTMODE))) {
		perror(allegfx_file);
		exit(EXIT_FAILURE);
	    }
    	    Fprintf(ofp,"/*\tSCCS Id: @(#)allegfx.h\t3.2\t96/05/17 */\n\n");
	    Fprintf(ofp,Dont_Edit_Code);	    
        } else if (util_mode == 'b') {
            bigtile_bmp = NULL;
        }
        
        while (filenum < 3) {        	
                int index = 0;

                if (!fopen_text_file(tilefiles[filenum], RDTMODE)) {
                        Fprintf(stderr,
                         "Cannot find file '%s'.\n", tilefiles[filenum]);
                        exit(EXIT_FAILURE);
                }
                
	    	if (util_mode == 'b' && !bigtile_bmp) {
	    	    /* Only make this once */
	            bigtile_bmp = create_bitmap(tile_x * TILES_PER_COL, tile_y * TILES_PER_ROW);
	    	}
                
                num_colors = colorsinmap;
                if (num_colors > MAXCOLORMAPSIZE) {
                        Fprintf(stderr, "too many colors (%d)\n", num_colors);
                        exit(EXIT_FAILURE);
                }
                if (!paletteflag) {
                        paletteptr = tibheader.palette;
                        for (i = 0; i < num_colors; i++) {
                                *paletteptr++ = ColorMap[CM_RED][i],
                                *paletteptr++ = ColorMap[CM_GREEN][i],
                                *paletteptr++ = ColorMap[CM_BLUE][i];
                        }
                        paletteflag++;
                }

                while (read_text_tile(pixels)) {
                        sprintf(filename, "%c%03d%s", 
                                (!filenum) ? 'm' : (filenum == 1) ? 'o' : 'e',
                                index, ".bmp");
			if (util_mode == 'i') {
                            Fprintf(ofp, "%04d (%s) : %s\n", 
                                tilecount, tilename(filenum+1,index), 
                                filename);
			} else if (util_mode == 'f') {
                                make_bitmap(pixels, tilecount, filenum < 2
                                            ,filename, FALSE);
			} else if (util_mode == 'b') {
                                make_bitmap(pixels, tilecount, filenum < 2
                                            ,filename, TRUE);
			}

                        tilecount++;     
                        index++;
                }
                
                if (util_mode == 'h') 
                    switch(filenum)
                    {
		            case 0:
		                Fprintf(ofp, "#define NUMBER_OF_MONS %d\n", 
		                        tilecount);
		                break;
		            case 1:
		                Fprintf(ofp, "#define NUMBER_OF_OBJS %d\n", 
		                        tilecount);
		                break;
		            case 2:
		                Fprintf(ofp, "#define NUMBER_OF_TILES %d\n", 
		                        tilecount);
		                break;
                    }

		if (util_mode == 'f')
		    Fprintf(stderr, "%d tiles processed from %s\n", 
					index, tilefiles[filenum]);

                (void) fclose_text_file();
                ++filenum;
        }

	/* Close file */
        if ((util_mode == 'i') || (util_mode == 'h'))
        	Fclose(ofp);
        else if (util_mode == 'f')
		Fprintf(stderr, "Total of %d bmp tiles written.\n", tilecount);
        else if (util_mode == 'b') {        	
        	save_bitmap(bigtile_file, bigtile_bmp, 0);
        	destroy_bitmap(bigtile_bmp);
        }

        exit(EXIT_SUCCESS);
        /*NOTREACHED*/
        return 0;
}


